// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: NetNode.java
// Classes: NetNode
// Original Author: ics125b spring 1996
// $Id: NetNode.java,v 1.11 1997/01/03 00:33:42 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.io.*;
import java.util.*;

/** This class models a node in our underlying connected graph
 * model. Nodes have ports that are their connection points to other
 * nodes. Arcs from one port to another.
 *
 * @see NetArc
 * @see NetPort
 */

public class NetNode extends NetPrimitive {

  /** A list of the ports on this node */
  protected NetPort portList[];
  protected int numPorts;

  /** Nodes may have a list of predefined Perspective's that give the
   * node different looks. Needs-More-Work: This code is not fully
   * operational right now. */
  protected Vector perspectiveList = new Vector(1);


  /** Construct a new node from the given default node and number of
   * ports. The attributes of the default node will be used if they
   * are not overridden in this node (i.e., nodes have attributes and
   * there is a virual copy relationship between some nodes). */
  public NetNode(NetNode deft, int nPorts) {
    numPorts = nPorts;
    portList = new NetPort[nPorts];
  }

  /** Construct a new NetNode with no default attributes and no ports. */
  public NetNode() { this(null, 0); }

  /** Remove this node from the underling connected graph model. */
  public void dispose() {
    System.out.println("disposing: " + toString());
    for (int i = 0; i < numPorts; ++i) {
      portList[i].dispose();
    }
  }

  /** Returns the attribute table of the node. */
  public Object getAttributes() { return null; }


  /** Usually when nodes are created it is deon through newInstance
   *  and there is no chance to supply a default node or to connect
   *  this node to some other application level object. So after a
   *  node is constructed initialize is called to supply that
   *  information. <p>
   *
   * Needs-More-Work: what is the class protocol design here? */
  public void initialize(NetNode default_node, Object model) { }

  /** Add a perspective to my list of predefined perspectives. */
  public void addPerspective(Perspective p) {
    perspectiveList.addElement(p);    
  }

  /** By default the zeroth perspective will be used */
  public Perspective perspectiveFor(String persName) {
    if (perspectiveList.size() == 0) return null;
    else return (Perspective) perspectiveList.elementAt(0);
  }

  /** returns the perspectiveList */
  Vector getPerspectiveList() { return perspectiveList; }

  /** NetNode's can process events. If the Editor does not want to
   * process an Event and the Editor's current Mode does not want to
   * process it, then the Event is passed to the NetNode under the
   * mouse, if there is one. This allows nodes to react to user input
   * to do application specific actions. By default all of these
   * event handlers do nothing and return false. By convention,
   * event handlers in subclasses should call the handler in their
   * superclass if the choose not to handle an event. */

  
  /** This event handler is defined to call editNode when the user
   * clicks the right-hand mouse button on a Perspective. */
  public boolean mouseUp(Event e, int x, int y) {
    if (e.modifiers == Event.CTRL_MASK) {
      Editor ce = Globals.curEditor();
      ActionEditNode act = new ActionEditNode(this);
      ce.executeAction(act, e);
      return true;
    }
    return false;
  }

  /** Pop up a meny showing a list of options/commands for this
   * node. This is not currently availible because AWT 1.0.2 does not
   * support pop up menus. */
  public void popNodeMenu() {
    System.out.println("now showing a menu of node options");
  }

  /** Edit this NetNode in some application specific way. Typically
   * open a dialog box to edit some attributes of the
   * node. Needs-More-Work: shuold provide a default attribute editor
   * for my system of node attributes. */
  public void editNode() { System.out.println("Now editing node"); }

  /** Do some application specific action just after this node is
   * connected to another node. the arguments contain some info about
   * what ports were connected. */
  public void postConnect(NetNode anotherNode,
			  NetPort myPort, NetPort otherPort) { }
  

  /** Do some application specific action just after this node is
   * disconnected from another node. the arguments contain some info
   * about what ports were connected. */
  public void postDisconnect(NetNode anotherNode,
			     NetPort myPort, NetPort otherPort) { }

  /** Allow foir application specific rules about which nodes can be
   * connected to which other nodes. This is called from the NetPort,
   * so the port has first say as to whether it can be connected to
   * some other port. NetPort.canConnectTo() just calls
   * NetNode.canConnectTo(). By default anything can be connected to
   * anything. */
  public boolean canConnectTo(NetNode otherNode,
			      NetPort otherPort, NetPort myPort) {
    return true;
  }

  /** Do some application specific actions after the node is placed in
   * a drawing area. */
  public void postPlacement(Editor ed) { }


} /* end class NetNode */








